
import gconf

import gtk
try :
    import hildon
except :
    hildon = False

class Configuration :

    def __init__ ( self , type ) :
        self._type = type
        self.read()

    def read ( self ) :
        self._client = gconf.client_get_default()

        self.homedir = self._client.get_string( "/apps/wifihood/basedir" ) or "/home/user/MyDocs"
        self.dbname = self._client.get_string( "/apps/wifihood/dbname" ) or "wifiscanner.db"
        self.mapsdir = self._client.get_string( "/apps/wifihood/maps" ) or "/home/user/MyDocs/.maps"
        self.mapclass = self._client.get_string( "/apps/wifihood/maptype" ) or "OpenStreetMap I"

        self.scan_period = self._client.get_int( "/apps/wifihood/scan-period" ) or 5000

        self.store_log = self._client.get_bool( "/apps/wifihood/store-logfile" )

        self.use_mapper = self._client.get_bool( "/apps/wifihood/use-mapper" )
        self.store_gps = self._client.get_bool( "/apps/wifihood/store-gps" )
        if self.use_mapper :
            # FIXME : This will reset the stored default
            self.store_gps = False
            self.lat = self._client.get_float( "/apps/maemo/maemo-mapper/center_latitude" )
            self.lon = self._client.get_float( "/apps/maemo/maemo-mapper/center_longitude" )
            self.zoom = self._client.get_int( "/apps/maemo/maemo-mapper/zoom" )
        else :
            self.lat = self._client.get_float( "/apps/wifihood/latitude" )
            self.lon = self._client.get_float( "/apps/wifihood/longitude" )
            self.zoom = self._client.get_int( "/apps/wifihood/map-zoom" )

        if self._type == 'scanner' :
            self.zoom = self._client.get_int( "/apps/wifihood/%s-zoom" % self._type )

        if self.lat == 0.0 and self.lon == 0.0 :
            self.lat , self.lon = 40.416 , -3.683
        if self.zoom == 0 :
            self.zoom = 15

    def set_latlon ( self , ( lat , lon ) ) :
        self._client.set_float( "/apps/wifihood/latitude" , lat )
        self._client.set_float( "/apps/wifihood/longitude" , lon )
        self.lat , self.lon = lat , lon

    def set_zoom ( self , zoom ) :
        self._client.set_int( "/apps/wifihood/%s-zoom" % self._type , zoom )
        self.zoom = zoom

    def save ( self , widget , event ) :
        self._client.set_string( "/apps/wifihood/basedir" , self.homedir )
        self._client.set_string( "/apps/wifihood/dbname" , self.dbname )
        self._client.set_string( "/apps/wifihood/maps" , self.mapsdir )
        self._client.set_string( "/apps/wifihood/maptype" , self.mapclass )

        self._client.set_int( "/apps/wifihood/scan-period" , self.scan_period )
        self._client.set_bool( "/apps/wifihood/store-logfile" , self.store_log )
        self._client.set_bool( "/apps/wifihood/use-mapper" , self.use_mapper )
        self._client.set_bool( "/apps/wifihood/store-gps" , self.store_gps )

        self.set_latlon( ( self.lat , self.lon ) )
        self.set_zoom( self.zoom )

class AbstractSettingsWindow :

    def __init__ ( self , config , handler ) :
        self.set_title( "Wifihood Settings" )

        scrollwin = self.MainArea()
        scrollwin.show()
        self.add(scrollwin)

        vbox = gtk.VBox(False, 0)
        vbox.show()
        scrollwin.add_with_viewport( vbox )


        dbframe = gtk.Frame( label="Database" )
        dbframe.set_label_align(0 , 0.1)
        dbframe.show()
        vbox.pack_start(dbframe, True, True, 0)

        database = gtk.Table(2, 2, False)
        database.show()
        dbframe.add(database)

        dblabel = gtk.Label( "Home directory" )
        dblabel.show()
        database.attach(dblabel, 0, 1, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        dbvalue = self.Entry()
        dbvalue.connect( "changed" , self.entry_cb , config , "basedir" )
        dbvalue.set_text( config.homedir )
        dbvalue.show()
        database.attach(dbvalue, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        dblabel = gtk.Label( "Database name" )
        dblabel.show()
        database.attach(dblabel, 0, 1, 1, 2, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        dbvalue = self.Entry()
        dbvalue.connect( "changed" , self.entry_cb , config , "dbname" )
        dbvalue.set_text( config.dbname )
        dbvalue.show()
        database.attach(dbvalue, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, 0, 0, 5)


        mapsframe = gtk.Frame( label="Maps" )
        mapsframe.set_label_align(0 , 0.1)
        mapsframe.show()
        vbox.pack_start(mapsframe, True, True, 0)

        maps = gtk.Table(2, 2, False)
        maps.show()
        mapsframe.add(maps)

        button = self.CheckButton()
        button.set_label( "Use OpenStreet maps" )
        button.set_active( 1 )
        button.show()
        maps.attach(button, 0, 1, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)

        zoomlevel = self.ZoomButton( "Zoom level" , config , handler )
        zoomlevel.show()
        maps.attach(zoomlevel, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 5)


        gpsframe = gtk.Frame( label="Coordinates" )
        gpsframe.set_label_align(0 , 0.1)
        gpsframe.show()
        vbox.pack_start(gpsframe, True, True, 0)

        gps = gtk.Table(2, 2, False)
        gps.show()
        gpsframe.add(gps)

        button = self.CheckButton()
        button.set_label( "Take initial coordinates from maemo-mapper" )
        button.connect( "toggled" , self.checkbutton_cb , config , "use-mapper" )
        button.set_active( config.use_mapper )
        button.show()
        gps.attach(button, 0, 2, 0, 1, gtk.EXPAND|gtk.FILL) #, 0, 0, 5)

        button = self.CheckButton()
        button.set_label( "Store changes in coordinates" )
        button.connect( "toggled" , self.checkbutton_cb , config , "store-gps" )
        button.set_active( config.store_gps )
        button.show()
        gps.attach(button, 0, 2, 1, 2, gtk.EXPAND|gtk.FILL) #, 0, 0, 5)


        dataframe = gtk.Frame( label="Data gathering" )
        dataframe.set_label_align(0 , 0.1)
        dataframe.show()
        vbox.pack_start(dataframe, True, True, 0)

        datatable = gtk.Table(2, 2, False)
        datatable.show()
        dataframe.add(datatable)

        scanlabel = gtk.Label( "Scanning interval" )
        scanlabel.show()
        datatable.attach(scanlabel, 0, 1, 0, 1, gtk.EXPAND|gtk.FILL)

        scanvalue = self.Entry()
        scanvalue.connect( "changed" , self.int_cb , config , "scan-period" , 1000 )
        scanvalue.set_text( "%s" % ( float(config.scan_period) / 1000 ) )
        scanvalue.show()
        datatable.attach(scanvalue, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL)

        button = self.CheckButton()
        button.set_label( "Write full logfile" )
        button.connect( "toggled" , self.checkbutton_cb , config , "store-logfile" )
        button.set_active( config.store_log )
        button.show()
        datatable.attach(button, 0, 2, 1, 2, gtk.EXPAND|gtk.FILL)

        self.show()

    def entry_cb ( self , entry , config , keyword ) :
        config._client.set_string( "%s/%s" % ( "/apps/wifihood" , keyword ) , entry.get_text() )

    def int_cb ( self , entry , config , keyword , scale=1 ) :
        config._client.set_int( "%s/%s" % ( "/apps/wifihood" , keyword ) , int( scale * float( entry.get_text() ) ) )

    def checkbutton_cb ( self , button , config , keyword ) :
        config._client.set_bool( "%s/%s" % ( "/apps/wifihood" , keyword ) , button.get_active() )

if hildon :

  class ZoomDialog ( hildon.TouchSelector ) :

    def __init__ ( self , config , handler ) :
        hildon.TouchSelector.__init__( self )
        self.handler = handler

        zooms = gtk.ListStore(str)

        active = index = 0
        for zoom in range(8,19) :
            iter = zooms.append()
            zooms.set( iter , 0 , "%2d" % zoom )
            if zoom == config.zoom :
                active = index
            index += 1

        column = self.append_text_column( zooms , True )

        # NOTE : with text=True, we must use 1 instead of 0
        self.set_active( 0 , active )

    def zoomdialog ( self , widget , conf ) :
        newzoom = int( widget.get_selector().get_current_text() )
        if self.handler : self.handler( newzoom )
        conf.set_zoom( newzoom )

  class SettingsWindow ( hildon.StackableWindow , AbstractSettingsWindow ) :

    def __init__ ( self , config , handler=None ) :
        hildon.StackableWindow.__init__( self )
        AbstractSettingsWindow.__init__( self , config , handler )

    def ZoomButton ( self , text , config , handler ) :
        selector = ZoomDialog( config , handler )
        button = hildon.PickerButton( gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
        button.set_title( text )
        button.set_selector( selector )
        button.connect_object( "value-changed", selector.zoomdialog , button , config )
        return button

    def MainArea ( self ) :
        return hildon.PannableArea()

    def Entry ( self ) :
        return hildon.Entry( gtk.HILDON_SIZE_FINGER_HEIGHT )

    def Button ( self , text , value ) :
        return hildon.Button( gtk.HILDON_SIZE_THUMB_HEIGHT , hildon.BUTTON_ARRANGEMENT_HORIZONTAL , text , "%s" % value )

    def CheckButton ( self ) :
        return hildon.CheckButton( gtk.HILDON_SIZE_FINGER_HEIGHT )

else :

  class ZoomDialog ( gtk.Dialog ) :

    def __init__ ( self , config , handler , labelsetter=None ) :
        gtk.Dialog.__init__( self , "Select zoom level",
                             None,
                             gtk.DIALOG_MODAL,
                             ( gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
                               gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT
                               )
                             )

        zooms = gtk.ListStore(int)
        combo = gtk.ComboBox( zooms )

        for zoom in range(8,19) :
            iter = zooms.append()
            zooms.set( iter , 0 , zoom )
            if zoom == config.zoom :
                combo.set_active_iter( iter )

        cell = gtk.CellRendererText()
        combo.pack_start(cell, True)
        combo.add_attribute(cell, 'text', 0)

        self.vbox.pack_start(combo , True, True, 0)

        self.connect_object( "response", self.response , combo , config , handler , labelsetter )

    def response ( self , combo , response  , config , handler , labelsetter ) :
        if response == gtk.RESPONSE_ACCEPT :
            item = combo.get_active_iter()
            model = combo.get_model()
            newzoom = model.get(item,0)[0]
            if labelsetter : labelsetter( newzoom )
            if handler : handler( newzoom )
            config.set_zoom( newzoom )
        self.destroy()

  class SettingsWindow ( gtk.Window , AbstractSettingsWindow ) :

    def __init__ ( self , config , handler=None ) :
        gtk.Window.__init__( self )
        AbstractSettingsWindow.__init__( self , config , handler )
        self.connect_object("delete_event", config.save , self )

    def MainArea ( self ) :
        scrollwin = gtk.ScrolledWindow()
        scrollwin.set_size_request(-1, 260)
        scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        return scrollwin

    def Entry ( self ) :
        return gtk.Entry()

    def Button ( self , label=None ) :
        return gtk.Button( label )

    def zoomdialog ( self , widget , config , handler ) :
        dialog = ZoomDialog( config , handler , widget.set_value )
        dialog.show_all()

    def ZoomButton ( self , text , config , handler ) :
        class _button ( gtk.Button ) :
            def __init__ ( self , text , value ) :
                gtk.Button.__init__( self )
                self._text = text
                self.set_value( value )
            def set_value ( self , value ) :
                self._value = value
                self.set_label( "%s -- %s" % ( self._text , self._value ) )
        button = _button( text , config.zoom )
        button.connect_object( "clicked", self.zoomdialog , button , config , handler )
        return button

    def CheckButton ( self ) :
        return gtk.CheckButton()

if __name__ == "__main__" :
    config = Configuration( 'scanner' )
    window = SettingsWindow( config )
    window.connect("delete_event", gtk.main_quit, None)
    window.show()
    gtk.main()
